@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 #--
 # Copyright 2006, 2007 by Chad Fowler, Rich Kilmer, Jim Weirich, Eric Hodel
 # and others.
@@ -6,21 +7,19 @@
 #++
 
 # Make sure rubygems isn't already loaded.
-if ENV['RUBYOPT'] and defined? Gem then
+if ENV['RUBYOPT'] or defined? Gem
   ENV.delete 'RUBYOPT'
 
   require 'rbconfig'
-  config = defined?(RbConfig) ? RbConfig : Config
+  cmd = [RbConfig.ruby, 'setup.rb', *ARGV].compact
+  cmd[1,0] = "--disable-gems"
 
-  ruby = File.join config::CONFIG['bindir'], config::CONFIG['ruby_install_name']
-  ruby << config::CONFIG['EXEEXT']
-
-  exec(ruby, 'setup.rb', *ARGV)
+  exec(*cmd)
 end
 
 Dir.chdir File.dirname(__FILE__)
 
-$:.unshift 'lib'
+$:.unshift File.expand_path('lib')
 require 'rubygems'
 require 'rubygems/gem_runner'
 require 'rubygems/exceptions'
@@ -28,7 +27,9 @@ require 'rubygems/exceptions'
 Gem::CommandManager.instance.register_command :setup
 
 args = ARGV.clone
-
+if ENV["GEM_PREV_VER"]
+  args = [ '--previous-version', ENV["GEM_PREV_VER"] ] + args
+end
 args.unshift 'setup'
 
 begin
@@ -36,4 +37,3 @@ begin
 rescue Gem::SystemExitException => e
   exit e.exit_code
 end
-
