@@ -1,10 +1,11 @@
+# frozen_string_literal: true
 require 'rubygems/test_case'
 require 'rubygems/commands/which_command'
 
 class TestGemCommandsWhichCommand < Gem::TestCase
-
   def setup
     super
+    Gem::Specification.reset
     @cmd = Gem::Commands::WhichCommand.new
   end
 
@@ -21,22 +22,8 @@ class TestGemCommandsWhichCommand < Gem::TestCase
     assert_equal '', @ui.error
   end
 
-  def test_execute_one_missing
-    util_foo_bar
-
-    @cmd.handle_options %w[foo_bar missing]
-
-    use_ui @ui do
-      @cmd.execute
-    end
-
-    assert_equal "#{@foo_bar.full_gem_path}/lib/foo_bar.rb\n", @ui.output
-    assert_match %r%Can't find ruby library file or shared library missing\n%,
-                 @ui.error
-  end
-
-  def test_execute_missing
-    @cmd.handle_options %w[missing]
+  def test_execute_directory
+    @cmd.handle_options %w[directory]
 
     use_ui @ui do
       assert_raises Gem::MockGemUi::TermError do
@@ -45,22 +32,53 @@ class TestGemCommandsWhichCommand < Gem::TestCase
     end
 
     assert_equal '', @ui.output
-    assert_match %r%Can't find ruby library file or shared library missing\n%,
+    assert_match %r{Can.t find Ruby library file or shared library directory\n},
+                 @ui.error
+  end
+
+  def test_execute_one_missing
+    # TODO: this test fails in isolation
+
+    util_foo_bar
+
+    @cmd.handle_options %w[foo_bar missinglib]
+
+    use_ui @ui do
+      assert_raises Gem::MockGemUi::TermError do
+        @cmd.execute
+      end
+    end
+
+    assert_equal "#{@foo_bar.full_gem_path}/lib/foo_bar.rb\n", @ui.output
+    assert_match %r{Can.t find Ruby library file or shared library missinglib\n},
+                 @ui.error
+  end
+
+  def test_execute_missing
+    @cmd.handle_options %w[missinglib]
+
+    use_ui @ui do
+      assert_raises Gem::MockGemUi::TermError do
+        @cmd.execute
+      end
+    end
+
+    assert_equal '', @ui.output
+    assert_match %r{Can.t find Ruby library file or shared library missinglib\n},
                  @ui.error
   end
 
   def util_foo_bar
-    files = %w[lib/foo_bar.rb Rakefile]
-    @foo_bar = quick_spec 'foo_bar' do |gem|
+    files = %w[lib/foo_bar.rb lib/directory/baz.rb Rakefile]
+    @foo_bar = util_spec 'foo_bar' do |gem|
       gem.files = files
     end
+    install_specs @foo_bar
 
     files.each do |file|
-      filename = @foo_bar.full_gem_path + "/#{file}"
-      FileUtils.mkdir_p File.dirname(filename)
+      filename = File.join(@foo_bar.full_gem_path, file)
+      FileUtils.mkdir_p File.dirname filename
       FileUtils.touch filename
     end
   end
-
 end
-
